/*
 * UserHandler.cpp
 *
 *  Created on: Apr 22, 2011
 *      Author: johnsverre
 */

#include "UserHandler.h"
#include "User.h"
#include <iostream>
#include <XnCppWrapper.h>
#include <boost/thread.hpp>
#include "DecisionMaker.h"

using namespace xn;

UserHandler::UserHandler() {
	this->decisionMaker = new DecisionMaker();
	this->door = new Door();
}

UserHandler::~UserHandler() {
	delete this->decisionMaker;
	delete this->door;
}


void UserHandler::addUser(User u){

	if(u.getUserID() == 0)
		return;
	cout << "adding user.. \n" << u.toString() << endl;
	users.push_back(u);
}


User UserHandler::getUser(int index){
	if(!users.empty() && users.size() > index){
		return users.at(index);
	}
	return User(0);
}

void UserHandler::removeUser(XnUserID userid){
	if(users.empty()) return;

	int size = users.size();

	for (int i = 0;i<size;i++){
		if(users[i].getUserID() == userid){
			cout << "removing user.. \n" << users[i].toString() << endl;
			users.erase(users.begin()+i);
			return;
		}
	}
}

void UserHandler::runUserCalculations(){
	if(users.empty()) return;

	int size = users.size();

	for (int i = 0;i<size;i++){
		users[i].calcFeatures();
	}
}

void UserHandler::updateUserPoints(){
	extern UserGenerator g_UserGenerator;

	if(users.empty()) return;

	int size = users.size();

	for (int i = 0;i<size;i++){
		XnUserID uid = users[i].getUserID();
		users[i].addUserPoints();
	}
}

int UserHandler::getUserCount() const{
	return users.size();
}

void UserHandler::inferUserIntentions(XnUserID &openingDoor){
	if(users.empty()) return;

	int size = users.size();

	bool openDoor = false;

	openingDoor = 0;

	for (int i = 0;i<size;i++){
		bool positiveIntention = decisionMaker->evaluateUser(users[i]);
		int nPositive = users[i].getPositiveIndications();
		if(positiveIntention){
			if(nPositive > 3){ // let user have multiple positive indications before opening
				openDoor = true;
				openingDoor = users[i].getUserID();
				//mark open request
				lastOpenRequest = time(NULL);
			}else{
				int n = users[i].getPositiveIndications();
				nPositive++;
				users[i].setPositiveIndications(nPositive);
			}
		}else{
			nPositive--;
			users[i].setPositiveIndications(nPositive);
		}
	}

	if(openDoor && this->door->isClosed()){
		boost::thread workerThread(&Door::openDoor, this->door);
	}

}

void UserHandler::checkDoorTime(){
	if(difftime(time(NULL), lastOpenRequest) > 5) // door has been open in five seconds
		boost::thread workerThread(&Door::closeDoor, this->door);
}


/*
 *
 * Manual overrides
 * Open and close door
 * */
void UserHandler::openDoor(){
	if(!this->door->isOpen())
		boost::thread workerThread(&Door::openDoor, this->door);
		lastOpenRequest = time(NULL);
}
void UserHandler::closeDoor(){
	if(!this->door->isClosed())
		boost::thread workerThread(&Door::closeDoor, this->door);
}
