package net.fosstveit.sliding.main;

import java.util.ArrayList;

import be.ac.ulg.montefiore.run.jahmm.Hmm;
import be.ac.ulg.montefiore.run.jahmm.ObservationInteger;
import be.ac.ulg.montefiore.run.jahmm.OpdfInteger;
import be.ac.ulg.montefiore.run.jahmm.ViterbiCalculator;

/**
 * @author Håvar Aambø Fosstveit
 * 
 */
public class DecisionManager {
	
	private Hmm<ObservationInteger> hmm;
	
	private int[] closeArea = {51, 52, 53, 54, 59, 60, 61, 62}; // Gridcells in the "close" area
	
	public DecisionManager(Hmm<ObservationInteger> hmm) {
		this.hmm = hmm;
	}
	
	public boolean shallWeOpenDoor(Person p) {
		ArrayList<ObservationInteger> obs = p.getObservations();
		
		boolean open = isStateInCloseArea(getNextState(obs));
		
		return open;
	}
	
	private ArrayList<Integer> getNextState(ArrayList<ObservationInteger> observations) {
		int[] sequence = new ViterbiCalculator(observations, hmm).stateSequence();
		
		int lastState = sequence[sequence.length - 1];
		
		int nextState = 0;
		double prob = 0.0;
		
		for(int i = 0; i < hmm.nbStates(); i++) {
			double tmp = hmm.getAij(lastState, i);
			if (tmp > prob) {
				nextState = i;
				prob = tmp;
			}
		}
		
		OpdfInteger o = (OpdfInteger) hmm.getOpdf(nextState);
		ArrayList<Integer> observ = new ArrayList<Integer>();
		
		for(int i = 0; i < 64; i++) {
			if (o.probability(new ObservationInteger(i)) > 0.001) {
				observ.add(i);
			}
		}
		
		return observ;
	}
	
	private boolean isStateInCloseArea(ArrayList<Integer> states) {
		for(Integer i : states) {
			for(int j = 0; j < closeArea.length; j++) {
				if (i == closeArea[j]) {
					return true;
				}
			}
		}
		
		return false;
	}
}
