package net.fosstveit.sliding.main;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;

import be.ac.ulg.montefiore.run.jahmm.Hmm;
import be.ac.ulg.montefiore.run.jahmm.ObservationInteger;
import be.ac.ulg.montefiore.run.jahmm.OpdfIntegerFactory;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationIntegerReader;
import be.ac.ulg.montefiore.run.jahmm.io.ObservationSequencesReader;
import be.ac.ulg.montefiore.run.jahmm.learn.BaumWelchLearner;

/**
 * @author Håvar Aambø Fosstveit
 * 
 */
public class Sliding {

	private Hmm<ObservationInteger> hmm;
	private List<List<ObservationInteger>> v;
	private Hmm<ObservationInteger> learntHmm;

	private ArrayList<Person> persons;
	private DecisionManager decisionManager;

	public Sliding() {
		OpdfIntegerFactory factory = new OpdfIntegerFactory(64);
		hmm = new Hmm<ObservationInteger>(16, factory);
		v = new ArrayList<List<ObservationInteger>>();
		persons = new ArrayList<Person>();

		loadData();

		BaumWelchLearner bwl = new BaumWelchLearner();

		learntHmm = bwl.learn(hmm, v);

		decisionManager = new DecisionManager(learntHmm);

		run();
	}

	private void run() {
		int[] person;
		boolean running = true;

		while (running) {
			person = getInput();

			updatePerson(person[0], person[1]);

			for (Person p : persons) {
				if (decisionManager.shallWeOpenDoor(p)) {
					System.out.println("Open door!");
				}
			}
		}
	}

	private void updatePerson(int person, int observation) {
		
		boolean done = false;
		
		for (Person p : persons) {
			if (p.getId() == person) {
				p.addObservation(observation);
				done = true;
			}
		}
		
		if (!done) {
			Person p = new Person(person);
			p.addObservation(observation);
			persons.add(p);
		}
	}

	private int[] getInput() {
		int[] ret = new int[2];
		String line;

		BufferedReader reader = new BufferedReader(new InputStreamReader(
				System.in));
		try {
			line = reader.readLine();
			String[] num = line.split(" ");
			ret[0] = Integer.parseInt(num[0]);
			ret[1] = Integer.parseInt(num[1]);
			System.out.println(line);
		} catch (IOException e) {
			e.printStackTrace();
		}

		return ret;
	}

	public static void main(String[] args) {
		new Sliding();
	}

	private void loadData() {
		Reader reader;
		try {
			reader = new FileReader("hmm.seq"); // Training data
			ObservationIntegerReader oi = new ObservationIntegerReader(64);
			v = ObservationSequencesReader.readSequences(oi, reader);
			reader.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
